
package jess;
import java.util.ArrayList;
import java.io.*;

/** **********************************************************************
 * An interface for conflict resolution strategies. Implement this
 * interface, then pass the class name to (set-strategy).
 *<P>
 * (C) 1997 Ernest J. Friedman-Hill and Sandia National Laboratories
 * @author Ernest J. Friedman-Hill
 ********************************************************************** */

public interface Strategy {
    /**
     * To implement your own conflict resolution strategy, you write
     * this method. It should behave like Comparable.compare() (which see.)
     * Place the two activations in order, respecting salience.
     */

    int compare(Activation a1, Activation a2);
    
    /**
     * Return the name of this strategy
     * @return a display name for this strategy
     */
    
    String getName();
}

class breadth implements Strategy, Serializable {
    
    public int compare(Activation a1, Activation a2) {
        int s1 = a1.getRule().getSalience();
        int s2 = a2.getRule().getSalience();
        
        if (s1 != s2)
            return s2 - s1;
        
        int t1 = a1.getToken().getTime();
        int t2 = a2.getToken().getTime();
        return t1 - t2;
    }
    
    public String getName() { return "breadth"; }
}









